package com.moyoung.moyoung_ble_plugin.method;

import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.content.Context;
import android.content.Intent;
import android.util.Log;

import com.crrepa.ble.CRPBleClient;
import com.crrepa.ble.conn.CRPBleConnection;
import com.crrepa.ble.conn.CRPBleDevice;
import com.crrepa.ble.conn.bean.CRPAlarmClockInfo;
import com.crrepa.ble.conn.bean.CRPContactConfigInfo;
import com.crrepa.ble.conn.bean.CRPContactInfo;
import com.crrepa.ble.conn.bean.CRPDrinkWaterPeriodInfo;
import com.crrepa.ble.conn.bean.CRPFirmwareVersionInfo;
import com.crrepa.ble.conn.bean.CRPFutureWeatherInfo;
import com.crrepa.ble.conn.bean.CRPHandWashingPeriodInfo;
import com.crrepa.ble.conn.bean.CRPMessageInfo;
import com.crrepa.ble.conn.bean.CRPPeriodTimeInfo;
import com.crrepa.ble.conn.bean.CRPPhysiologcalPeriodInfo;
import com.crrepa.ble.conn.bean.CRPPillReminderInfo;
import com.crrepa.ble.conn.bean.CRPSedentaryReminderPeriodInfo;
import com.crrepa.ble.conn.bean.CRPSupportWatchFaceInfo;
import com.crrepa.ble.conn.bean.CRPTodayWeatherInfo;
import com.crrepa.ble.conn.bean.CRPUserInfo;
import com.crrepa.ble.conn.bean.CRPWatchFaceInfo;
import com.crrepa.ble.conn.bean.CRPWatchFaceLayoutInfo;
import com.crrepa.ble.conn.callback.CRPDeviceNewFirmwareVersionCallback;
import com.crrepa.ble.conn.callback.CRPDeviceWatchFaceCallback;
import com.crrepa.ble.conn.callback.CRPDeviceWatchFaceStoreCallback;
import com.crrepa.ble.conn.type.CRPBloodOxygenTimeType;
import com.crrepa.ble.conn.type.CRPDeviceLanguageType;
import com.crrepa.ble.conn.type.CRPDeviceVersionType;
import com.crrepa.ble.conn.type.CRPFirmwareUpgradeType;
import com.crrepa.ble.conn.type.CRPHeartRateType;
import com.crrepa.ble.conn.type.CRPHistoryDynamicRateType;
import com.crrepa.ble.conn.type.CRPMetricSystemType;
import com.crrepa.ble.conn.type.CRPMovementHeartRateStateType;
import com.crrepa.ble.conn.type.CRPMusicPlayerStateType;
import com.crrepa.ble.conn.type.CRPPastTimeType;
import com.crrepa.ble.conn.type.CRPStepsCategoryDateType;
import com.crrepa.ble.conn.type.CRPTempTimeType;
import com.crrepa.ble.conn.type.CRPTimeSystemType;
import com.crrepa.ble.conn.type.CRPWatchFaceType;
import com.moyoung.moyoung_ble_plugin.base.BaseChannelHandler;
import com.moyoung.moyoung_ble_plugin.base.BaseConnectionEventChannelHandler;
import com.moyoung.moyoung_ble_plugin.common.GsonUtils;
import com.moyoung.moyoung_ble_plugin.event.ConnEveChlRegister;
import com.moyoung.moyoung_ble_plugin.model.BrightnessBean;
import com.moyoung.moyoung_ble_plugin.model.CheckFirmwareVersionBean;
import com.moyoung.moyoung_ble_plugin.model.ContactBean;
import com.moyoung.moyoung_ble_plugin.model.DeviceLanguageBean;
import com.moyoung.moyoung_ble_plugin.model.FirmwareVersionBean;
import com.moyoung.moyoung_ble_plugin.model.MaxHeartRateBean;
import com.moyoung.moyoung_ble_plugin.model.MenstrualCycleBean;
import com.moyoung.moyoung_ble_plugin.model.OTABean;
import com.moyoung.moyoung_ble_plugin.model.PeriodTimeBean;
import com.moyoung.moyoung_ble_plugin.model.PillReminderBean;
import com.moyoung.moyoung_ble_plugin.model.WatchFaceIDBean;
import com.moyoung.moyoung_ble_plugin.model.WatchFaceStoreBean;
import com.moyoung.moyoung_ble_plugin.model.WatchFaceStoreListBean;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import androidx.annotation.NonNull;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;

import static com.moyoung.moyoung_ble_plugin.ChannelNames.EVENT_CONTACT_AVATAR;
import static com.moyoung.moyoung_ble_plugin.ChannelNames.EVENT_ECG;
import static com.moyoung.moyoung_ble_plugin.ChannelNames.EVENT_FILE_TRANS;
import static com.moyoung.moyoung_ble_plugin.ChannelNames.EVENT_WF_FILE_TRANS;

public class ConnectionMethodHandler extends BaseChannelHandler implements MethodChannel.MethodCallHandler {
    private final static String TAG = ConnectionMethodHandler.class.getSimpleName();

    private final ConnEveChlRegister connEveChlRegister;
    private CRPBleDevice bleDevice;
    private CRPBleConnection bleConnection;

    public ConnectionMethodHandler(ConnEveChlRegister connEveChlRegister) {
        this.connEveChlRegister = connEveChlRegister;
    }

    @Override
    public void bindActivity(Activity activity) {
        super.bindActivity(activity);
        connEveChlRegister.bindActivity(this.activity);
    }

    public void unbindChannel(MethodChannel methodChannel) {
        methodChannel.setMethodCallHandler(null);
        connEveChlRegister.unregister();
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        if (!checkBluetoothEnable()) {
            return;
        }

        if ("connect".equals(call.method)) {
            bleConnection = connect(call, activity.getApplicationContext());
            return;
        }

        if (bleConnection == null) {
            Log.v(TAG, "BleConnection is disconnected. Please connect first!");
            return;
        }

        switch (call.method) {
            case "disconnect":
                disconnect(result);
                break;
            case "isConnected":
                isConnected(call, result);
                break;
            case "queryTime":
                bleConnection.syncTime();
                break;
            case "sendTimeSystem":
                sendTimeSystem(call);
                break;
            case "queryFirmwareVersion":
                queryFirmwareVersion(result);
                break;
            case "queryTimeSystem":
                queryTimeSystem(result);
                break;
            case "checkFirmwareVersion":
                checkFirmwareVersion(call, result);
                break;
            case "queryDeviceBattery":
                queryDeviceBattery();
                break;
            case "subscribeDeviceBattery":
                subscribeDeviceBattery();
                break;
            case "sendUserInfo":
                sendUserInfo(call);
                break;
            case "sendStepLength":
                sendStepLength(call);
                break;
            case "sendTodayWeather":
                sendTodayWeather(call);
                break;
            case "sendFutureWeather":
                sendFutureWeather(call);
                break;
            case "querySteps":
                querySteps();
                break;
            case "queryHistorySteps":
                queryHistorySteps(call);
                break;
            case "queryStepsDetail":
                queryStepsDetail(call);
                break;
            case "querySleep":
                querySleep();
                break;
            case "queryRemSleep":
                queryRemSleep();
                break;
            case "queryHistorySleep":
                queryHistorySleep(call);
                break;
            case "startOTA":
                startOTA(call);
                break;
            case "abortOTA":
                abortOTA(call);
                break;
            case "queryDeviceOtaStatus":
                queryDeviceOtaStatus(result);
                break;
            case "queryHsOtaAddress":
                queryHsOtaAddress(result);
                break;
            case "enableHsOta":
                enableHsOta();
                break;
            case "queryOtaType":
                queryOtaType(result);
                break;
            case "queryUnitSystem":
                queryUnitSystem(result);
                break;
            case "sendUnitSystem":
                sendUnitSystem(call);
                break;
            case "sendQuickView":
                sendQuickView(call);
                break;
            case "queryQuickView":
                queryQuickView(result);
                break;
            case "sendQuickViewTime":
                sendQuickViewTime(call);
                break;
            case "queryQuickViewTime":
                queryQuickViewTime(result);
                break;
            case "sendGoalSteps":
                sendGoalSteps(call);
                break;
            case "queryGoalSteps":
                queryGoalSteps(result);
                break;
            case "sendDisplayWatchFace":
                sendDisplayWatchFace(call);
                break;
            case "queryDisplayWatchFace":
                queryDisplayWatchFace(result);
                break;
            case "queryWatchFaceLayout":
                queryWatchFaceLayout(result);
                break;
            case "sendWatchFaceLayout":
                sendWatchFaceLayout(call);
                break;
            case "querySupportWatchFace":
                querySupportWatchFace(result);
                break;
            case "queryWatchFaceStore":
                queryWatchFaceStore(call, result);
                break;
            case "queryWatchFaceOfID":
                queryWatchFaceOfID(call, result);
                break;
            case "sendWatchFace":
                sendWatchFace(call);
                break;
            case "sendAlarm":
                sendAlarm(call);
                break;
            case "queryAllAlarm":
                queryAllAlarm(result);
                break;
            case "queryLastDynamicRate":
                queryLastDynamicRate(call);
                break;
            case "enableTimingMeasureHeartRate":
                enableTimingMeasureHeartRate(call);
                break;
            case "disableTimingMeasureHeartRate":
                disableTimingMeasureHeartRate();
                break;
            case "queryTimingMeasureHeartRate":
                queryTimingMeasureHeartRate(result);
                break;
            case "queryTodayHeartRate":
                queryTodayHeartRate(call);
                break;
            case "queryPastHeartRate":
                queryPastHeartRate();
                break;
            case "queryTrainingHeartRate":
                queryTrainingHeartRate();
                break;
            case "startMeasureOnceHeartRate":
                startMeasureOnceHeartRate();
                break;
            case "stopMeasureOnceHeartRate":
                stopMeasureOnceHeartRate();
                break;
            case "queryHistoryHeartRate":
                queryHistoryHeartRate();
                break;
            case "startMeasureBloodPressure":
                startMeasureBloodPressure();
                break;
            case "stopMeasureBloodPressure":
                stopMeasureBloodPressure();
                break;
            case "enableContinueBloodPressure":
                enableContinueBloodPressure();
                break;
            case "disableContinueBloodPressure":
                disableContinueBloodPressure();
                break;
            case "queryContinueBloodPressureState":
                queryContinueBloodPressureState();
                break;
            case "queryLast24HourBloodPressure":
                queryLast24HourBloodPressure();
                break;
            case "queryHistoryBloodPressure":
                queryHistoryBloodPressure();
                break;
            case "startMeasureBloodOxygen":
                startMeasureBloodOxygen();
                break;
            case "stopMeasureBloodOxygen":
                stopMeasureBloodOxygen();
                break;
            case "enableTimingMeasureBloodOxygen":
                enableTimingMeasureBloodOxygen(call);
                break;
            case "disableTimingMeasureBloodOxygen":
                disableTimingMeasureBloodOxygen();
                break;
            case "queryTimingBloodOxygenMeasureState":
                queryTimingBloodOxygenMeasureState();
                break;
            case "queryTimingBloodOxygen":
                queryTimingBloodOxygen(call);
                break;
            case "enableContinueBloodOxygen":
                enableContinueBloodOxygen();
                break;
            case "disableContinueBloodOxygen":
                disableContinueBloodOxygen();
                break;
            case "queryContinueBloodOxygenState":
                queryContinueBloodOxygenState();
                break;
            case "queryLast24HourBloodOxygen":
                queryLast24HourBloodOxygen();
                break;
            case "queryHistoryBloodOxygen":
                queryHistoryBloodOxygen();
                break;
            case "enterCameraView":
                enterCameraView();
                break;
            case "exitCameraView":
                exitCameraView();
                break;
            case "readDeviceRssi":
                readDeviceRssi();
                break;
            case "setECGChangeListener":
                setECGChangeListener(call);
                break;
            case "startECGMeasure":
                startECGMeasure();
                break;
            case "stopECGMeasure":
                stopECGMeasure();
                break;
            case "isNewECGMeasurementVersion":
                isNewECGMeasurementVersion(result);
                break;
            case "queryLastMeasureECGData":
                queryLastMeasureECGData();
                break;
            case "sendECGHeartRate":
                sendECGHeartRate(call);
                break;
            case "sendWatchFaceBackground":
                sendWatchFaceBackground(call);
                break;
            case "sendDeviceLanguage":
                sendDeviceLanguage(call);
                break;
            case "queryDeviceLanguage":
                queryDeviceLanguage(result);
                break;
            case "sendMessage":
                sendMessage(call);
                break;
            case "endCall":
                endCall();
                break;
            case "sendSedentaryReminder":
                sendSedentaryReminder(call);
                break;
            case "querySedentaryReminderPeriod":
                querySedentaryReminderPeriod(result);
                break;
            case "sendSedentaryReminderPeriod":
                sendSedentaryReminderPeriod(call);
                break;
            case "querySedentaryReminder":
                querySedentaryReminder(result);
                break;
            case "findDevice":
                findDevice();
                break;
            case "shutDown":
                shutDown();
                break;
            case "sendDoNotDisturbTime":
                sendDoNotDisturbTime(call);
                break;
            case "queryDoNotDisturbTime":
                queryDoNotDisturbTime(result);
                break;
            case "sendBreathingLight":
                sendBreathingLight(call);
                break;
            case "queryBreathingLight":
                queryBreathingLight(result);
                break;
            case "sendMenstrualCycle":
                sendMenstrualCycle(call);
                break;
            case "queryMenstrualCycle":
                queryMenstrualCycle(result);
                break;
            case "startFindPhone":
                startFindPhone();
                break;
            case "stopFindPhone":
                stopFindPhone();
                break;
            case "setPlayerState":
                setPlayerState(call);
                break;
            case "sendSongTitle":
                sendSongTitle(call);
                break;
            case "sendLyrics":
                sendLyrics(call);
                break;
            case "closePlayerControl":
                closePlayerControl();
                break;
            case "sendCurrentVolume":
                sendCurrentVolume(call);
                break;
            case "sendMaxVolume":
                sendMaxVolume(call);
                break;
            case "enableDrinkWaterReminder":
                enableDrinkWaterReminder(call);
                break;
            case "disableDrinkWaterReminder":
                disableDrinkWaterReminder();
                break;
            case "queryDrinkWaterReminderPeriod":
                queryDrinkWaterReminderPeriod(result);
                break;
            case "setMaxHeartRate":
                setMaxHeartRate(call);
                break;
            case "queryMaxHeartRate":
                queryMaxHeartRate(result);
                break;
            case "startTraining":
                startTraining(call);
                break;
            case "setTrainingState":
                setTrainingState(call);
                break;
            case "getProtocolVersion":
                getProtocolVersion(result);
                break;
            case "startMeasureTemp":
                startMeasureTemp();
                break;
            case "stopMeasureTemp":
                stopMeasureTemp();
                break;
            case "enableTimingMeasureTemp":
                enableTimingMeasureTemp();
                break;
            case "disableTimingMeasureTemp":
                disableTimingMeasureTemp();
                break;
            case "queryTimingMeasureTempState":
                queryTimingMeasureTempState(result);
                break;
            case "queryTimingMeasureTemp":
                queryTimingMeasureTemp(call);
                break;
            case "sendDisplayTime":
                sendDisplayTime(call);
                break;
            case "queryDisplayTime":
                queryDisplayTime(result);
                break;
            case "enableHandWashingReminder":
                enableHandWashingReminder(call);
                break;
            case "disableHandWashingReminder":
                disableHandWashingReminder();
                break;
            case "queryHandWashingReminderPeriod":
                queryHandWashingReminderPeriod(result);
                break;
            case "sendLocalCity":
                sendLocalCity(call);
                break;
            case "sendTempUnit":
                sendTempUnit(call);
                break;
            case "queryTempUnit":
                queryTempUnit();
                break;
            case "sendBrightness":
                sendBrightness(call);
                break;
            case "queryBrightness":
                queryBrightness(result);
                break;
            case "queryBtAddress":
                queryBtAddress(result);
                break;
            case "checkSupportQuickContact":
                checkSupportQuickContact(result);
                break;
            case "queryContactCount":
                queryContactCount(result);
                break;
            case "sendContact":
                sendContact(call);
                break;
            case "sendContactAvatar":
                sendContactAvatar(call);
                break;
            case "deleteContact":
                deleteContact(call);
                break;
            case "deleteContactAvatar":
                deleteContactAvatar(call);
                break;
            case "clearContact":
                clearContact();
                break;
            case "queryBatterySaving":
                queryBatterySaving();
                break;
            case "sendBatterySaving":
                sendBatterySaving(call);
                break;
            case "queryPillReminder":
                queryPillReminder(result);
                break;
            case "sendPillReminder":
                sendPillReminder(call);
                break;
            case "deletePillReminder":
                deletePillReminder(call);
                break;
            case "clearPillReminder":
                clearPillReminder();
                break;
            case "queryWakeState":
                queryWakeState(result);
                break;
            case "sendWakeState":
                sendWakeState(call);
                break;
            case "queryHistoryTraining":
                queryHistoryTraining();
                break;
            case "queryTraining":
                queryTraining(call);
                break;
            default:
                result.notImplemented();

        }
    }

    private void queryTraining(MethodCall call) {
        Integer id = call.argument("id");
        id = id == null ? -1 : id;
        Log.d(TAG, "" + id);
        bleConnection.queryTraining(id);
    }

    private void queryHistoryTraining() {
        bleConnection.queryHistoryTraining();
    }

    private void sendWakeState(MethodCall call) {
        Boolean enable = call.argument("enable");
        enable = enable != null && enable;
        Log.d(TAG, "" + enable);
        bleConnection.sendTapToWakeState(enable);
    }

    private void queryWakeState(MethodChannel.Result result) {
        bleConnection.queryTapToWakeState(result::success);
    }

    private void clearPillReminder() {
        bleConnection.clearPillReminder();
    }

    private void deletePillReminder(MethodCall call) {
        Integer id = call.argument("id");
        id = id == null ? -1 : id;
        Log.d(TAG, "" + id);
        bleConnection.deletePillReminder(id);
    }

    private void sendPillReminder(MethodCall call) {
        String jsonStr = call.argument("pillReminderInfo");
        Log.d(TAG, jsonStr);
        CRPPillReminderInfo pillReminderInfo = GsonUtils.get().fromJson(jsonStr,
                CRPPillReminderInfo.class);
        Log.d(TAG, "" + pillReminderInfo);
        bleConnection.sendPillReminder(pillReminderInfo);
    }

    private void queryPillReminder(MethodChannel.Result result) {
        bleConnection.queryPillReminder((supportCount, list) -> {
            PillReminderBean pillReminderBean = new PillReminderBean();
            pillReminderBean.setSupportCount(supportCount);
            pillReminderBean.setList(list);
            Log.d(TAG, supportCount + "");
            String jsonStr = GsonUtils.get().toJson(pillReminderBean, PillReminderBean.class);
            result.success(jsonStr);
        });
    }

    private void sendBatterySaving(MethodCall call) {
        Boolean enable = call.argument("enable");
        enable = enable != null && enable;
        Log.d(TAG, "" + enable);
        bleConnection.sendBatterySaving(enable);
    }

    private void queryBatterySaving() {
        bleConnection.queryBatterySaving();
    }

    private void clearContact() {
        bleConnection.clearContact();
    }

    private void deleteContact(MethodCall call) {
        Integer index = call.argument("index");
        if (index != null) {
            Log.d(TAG, index + "");
            bleConnection.deleteContact(index);
        }
    }

    private void deleteContactAvatar(MethodCall call) {
        Integer index = call.argument("index");
        if (index != null) {
            Log.d(TAG, index + "");
            bleConnection.deleteContactAvatar(index);
        }
    }

    private void sendContact(MethodCall call) {
        String jsonStr = call.argument("contactBean");
        Log.d(TAG, jsonStr);
        ContactBean contactBean = GsonUtils.get().fromJson(jsonStr,
                ContactBean.class);
        CRPContactInfo crpContactInfo = new CRPContactInfo(
                contactBean.getId(),
                contactBean.getWidth(),
                contactBean.getHeight(),
                contactBean.getAddress(),
                contactBean.getName(),
                contactBean.getNumber());
        Log.d(TAG, "" + crpContactInfo);
        bleConnection.sendContact(crpContactInfo);
    }

    private void sendContactAvatar(MethodCall call) {
        BaseConnectionEventChannelHandler lazyEveChlHandler
                = connEveChlRegister.getLazyEveChlHandler(EVENT_CONTACT_AVATAR);
        if (lazyEveChlHandler != null) {
            lazyEveChlHandler.setConnListener(bleConnection, call);
        }
    }

    private void queryContactCount(MethodChannel.Result result) {
        bleConnection.queryContactCount(result::success);
    }

    private void checkSupportQuickContact(MethodChannel.Result result) {
        Log.d(TAG, "checkSupportQuickContact: --------");
        bleConnection.checkSupportQuickContact(contactConfigInfo -> {
            Log.d(TAG, "checkSupportQuickContact: =========");
            String jsonStr = GsonUtils.get().toJson(contactConfigInfo, CRPContactConfigInfo.class);
            result.success(jsonStr);
        });
    }

    private void queryBtAddress(MethodChannel.Result result) {
        bleConnection.queryBtAddress(result::success);
    }

    private void queryBrightness(MethodChannel.Result result) {
        bleConnection.queryBrightness((current, max) -> {
            BrightnessBean brightness = new BrightnessBean(current,max);
            String jsonStr = GsonUtils.get().toJson(brightness, BrightnessBean.class);
            Log.i(TAG, "onBrightness: " + jsonStr);
            result.success(jsonStr);
        });
    }

    private void sendBrightness(MethodCall call) {
        Integer brightness = call.argument("brightness");
        brightness = brightness == null ? -1 : brightness;
        Log.d(TAG, brightness+"");
        bleConnection.sendBrightness(brightness);
    }

    private void queryTempUnit() {
        bleConnection.queryTempUnit();
    }

    private void sendTempUnit(MethodCall call) {
        Integer temp = call.argument("temp");
        temp = temp == null ? 1 : temp;
        Log.d("sendTempUnit", temp+"");
        bleConnection.sendTempUnit(temp.byteValue());
    }

    private void sendLocalCity(MethodCall call) {
        String city = call.argument("city");
        bleConnection.sendLocalCity(city);
    }

    private void queryHandWashingReminderPeriod(MethodChannel.Result result) {
        bleConnection.queryHandWashingReminderPeriod(crpHandWashingPeriodInfo -> {
            String jsonStr = GsonUtils.get().toJson(crpHandWashingPeriodInfo);
            result.success(jsonStr);
        });
    }

    private void disableHandWashingReminder() {
        bleConnection.disableHandWashingReminder();
    }

    private void enableHandWashingReminder(MethodCall call) {
        String jsonStr = call.argument("handWashingPeriodInfo");
        Log.d(TAG, jsonStr);
        CRPHandWashingPeriodInfo handWashingPeriodInfo = GsonUtils.get().fromJson(jsonStr,
                CRPHandWashingPeriodInfo.class);
        bleConnection.enableHandWashingReminder(handWashingPeriodInfo);
    }

    private void queryDisplayTime(MethodChannel.Result result) {
        bleConnection.queryDisplayTime(result::success);
    }

    private void sendDisplayTime(MethodCall call) {
        Integer time = call.argument("time");
        if (time != null && time % 5 == 0 && 0 < time && time <= 30) {
            Log.d(TAG,time+"");
            bleConnection.sendDisplayTime(time);
        }
    }

    private void queryTimingMeasureTemp(MethodCall call) {
        String tempTimeType = call.argument("tempTimeType");
        tempTimeType = tempTimeType == null ? "TODAY" : tempTimeType;
        Log.d(TAG, ""+CRPTempTimeType.valueOf(tempTimeType));
        bleConnection.queryTimingMeasureTemp(CRPTempTimeType.valueOf(tempTimeType));
    }

    private void queryTimingMeasureTempState(MethodChannel.Result result) {
        bleConnection.queryTimingMeasureTempState(timingTempState -> {
            String jsonStr = GsonUtils.get().toJson(timingTempState);
            result.success(jsonStr);
        });
    }

    private void disableTimingMeasureTemp() {
        bleConnection.disableTimingMeasureTemp();
    }

    private void enableTimingMeasureTemp() {
        bleConnection.enableTimingMeasureTemp();
    }

    private void stopMeasureTemp() {
        bleConnection.stopMeasureTemp();
    }

    private void startMeasureTemp() {
        bleConnection.startMeasureTemp();
    }

    private void setTrainingState(MethodCall call) {
        Integer training = call.argument("training");
        training = training == null ? CRPMovementHeartRateStateType.MOVEMENT_COMPLETE : training;
        Log.d(TAG, ""+training);
        bleConnection.setMovementState(training.byteValue());
    }

    private void startTraining(MethodCall call) {
        Integer type = call.argument("type");
        type = type == null ? 0 : type;
        Log.d(TAG, ""+type);
        bleConnection.startMovement(type.byteValue());
    }

    private void queryMaxHeartRate(MethodChannel.Result result) {
        bleConnection.queryMaxHeartRate((heartRate, enable) -> {
            MaxHeartRateBean maxHeartRate = new MaxHeartRateBean(heartRate,enable);
            String jsonStr = GsonUtils.get().toJson(maxHeartRate, MaxHeartRateBean.class);
            Log.i(TAG, "queryMaxHeartRate: "+ jsonStr);
            result.success(jsonStr);
        });
    }

    private void setMaxHeartRate(MethodCall call) {
        String jsonStr =  call.argument("maxHeartRateBean");
        MaxHeartRateBean maxHeartRate = GsonUtils.get().fromJson(jsonStr,
                MaxHeartRateBean.class);
        bleConnection.setMaxHeartRate(maxHeartRate.getHeartRate(), maxHeartRate.getEnable());
    }

    private void queryDrinkWaterReminderPeriod(MethodChannel.Result result) {
        bleConnection.queryDrinkWaterReminderPeriod(drinkWaterPeriodInfo -> {
            String jsonStr = GsonUtils.get().toJson(drinkWaterPeriodInfo);
            result.success(jsonStr);
        });
    }

    private void disableDrinkWaterReminder() {
        bleConnection.disableDrinkWaterReminder();
    }

    private void enableDrinkWaterReminder(MethodCall call) {
        String jsonStr = call.argument("drinkWaterPeriodInfo");
        Log.d(TAG, jsonStr);
        CRPDrinkWaterPeriodInfo drinkWaterPeriodInfo = GsonUtils.get().fromJson(jsonStr,
                CRPDrinkWaterPeriodInfo.class);
        Log.d(TAG, drinkWaterPeriodInfo + "");
        bleConnection.enableDrinkWaterReminder(drinkWaterPeriodInfo);
    }

    private void sendMaxVolume(MethodCall call) {
        Integer volume = call.argument("volume");
        volume = volume == null ? 0 : volume;
        Log.d(TAG, volume+"");
        bleConnection.sendMaxVolume(volume);
    }

    private void sendCurrentVolume(MethodCall call) {
        Integer volume = call.argument("volume");
        volume = volume == null ? 0 : volume;
        Log.d(TAG, volume+"");
        bleConnection.sendCurrentVolume(volume);
    }

    private void closePlayerControl() {
        bleConnection.closeMusicControl();
    }

    private void sendLyrics(MethodCall call) {
        bleConnection.sendLyrics(call.argument("lyrics"));
    }

    private void sendSongTitle(MethodCall call) {
        bleConnection.sendSongTitle(call.argument("title"));
    }

    private void setPlayerState(MethodCall call) {
        Integer type = call.argument("musicPlayerStateType");
        Log.d("setPlayerState", ""+type);
        type = type == null ? CRPMusicPlayerStateType.MUSIC_PLAYER_PAUSE : type;
        bleConnection.setMusicPlayerState(type.byteValue());
    }

    private void stopFindPhone() {
        bleConnection.stopFindPhone();
    }

    private void startFindPhone() {
        bleConnection.startFindPhone();
    }

    private void sendMenstrualCycle (MethodCall call) {
        String jsonStr = call.argument("menstrualCycleBean");
        Log.d(TAG, jsonStr);

        MenstrualCycleBean menstrualCycleBean = GsonUtils.get().fromJson(jsonStr,
                MenstrualCycleBean.class);
        Log.d(TAG, MenstrualCycleBean.convert(menstrualCycleBean)+"");
        bleConnection.sendPhysiologcalPeriod(MenstrualCycleBean.convert(menstrualCycleBean));
    }

    private void queryMenstrualCycle(MethodChannel.Result result) {
        bleConnection.queryPhysiologcalPeriod(physiologcalPeriodInfo -> {
            String jsonStr = GsonUtils.get().toJson(physiologcalPeriodInfo, CRPPhysiologcalPeriodInfo.class);
            Log.d(TAG, "queryMenstrualCycle: "+jsonStr);
            result.success(jsonStr);
        });
    }

    private void getProtocolVersion(MethodChannel.Result result) {
        String jsonStr = GsonUtils.get().toJson(bleConnection.getProtocolVersion());
        Log.d(TAG, jsonStr+"");
        result.success(jsonStr);
    }

    private void sendFutureWeather(MethodCall call) {
        String jsonStr = call.argument("futureWeatherInfo");
        Log.d(TAG, jsonStr);
        CRPFutureWeatherInfo futureWeatherInfo = GsonUtils.get().fromJson(jsonStr,
                CRPFutureWeatherInfo.class);
        bleConnection.sendFutureWeather(futureWeatherInfo);
    }

    private void sendTodayWeather(MethodCall call) {
        String jsonStr = call.argument("todayWeatherInfo");
        Log.d(TAG, jsonStr);
        CRPTodayWeatherInfo todayWeatherInfo = GsonUtils.get().fromJson(jsonStr,
                CRPTodayWeatherInfo.class);
        bleConnection.sendTodayWeather(todayWeatherInfo);
    }

    private void sendStepLength(MethodCall call) {
        Integer stepLength = call.argument("stepLength");
        stepLength = stepLength == null ? -1 : stepLength;
        bleConnection.sendStepLength(stepLength.byteValue());
    }

    private void sendUserInfo(MethodCall call) {
        String jsonStr = call.argument("userInfo");
        Log.d(TAG, jsonStr);
        CRPUserInfo userInfo = GsonUtils.get().fromJson(jsonStr, CRPUserInfo.class);
        bleConnection.sendUserInfo(userInfo);
    }

    private void sendTimeSystem(@NonNull MethodCall call) {
        Integer timeSystemType = call.argument("timeSystemType");
        timeSystemType = timeSystemType == null ? CRPTimeSystemType.TIME_SYSTEM_12 : timeSystemType;

        bleConnection.sendTimeSystem(timeSystemType.byteValue());
    }

    private void queryTimeSystem(MethodChannel.Result result) {
        bleConnection.queryTimeSystem(result::success);
    }

    private void queryDeviceBattery() {
        bleConnection.queryDeviceBattery();
    }

    private void subscribeDeviceBattery() {
        bleConnection.subscribeDeviceBattery();
    }

    private void queryFirmwareVersion(MethodChannel.Result result) {
        bleConnection.queryFrimwareVersion(result::success);
    }

    private void checkFirmwareVersion(MethodCall call, MethodChannel.Result result) {
        String jsonStr = call.argument("firmwareVersion");
        FirmwareVersionBean firmwareVersionBean=GsonUtils.get().fromJson(jsonStr, FirmwareVersionBean.class);

        if (bleConnection == null) {
            result.success(null);
            return;
        }

        bleConnection.checkFirmwareVersion(new CRPDeviceNewFirmwareVersionCallback() {
            @Override
            public void onNewFirmwareVersion(CRPFirmwareVersionInfo firmwareVersionInfo) {
                CheckFirmwareVersionBean firmwareVersionBean = new CheckFirmwareVersionBean();
                firmwareVersionBean.setIsLatestVersion(false);
                firmwareVersionBean.setFirmwareVersionInfo(firmwareVersionInfo);
                String jsonStr = GsonUtils.get().toJson(firmwareVersionBean);
                result.success(jsonStr);
            }

            @Override
            public void onLatestVersion() {
                CheckFirmwareVersionBean firmwareVersionBean = new CheckFirmwareVersionBean();
                firmwareVersionBean.setIsLatestVersion(true);
                String jsonStr = GsonUtils.get().toJson(firmwareVersionBean);
                result.success(jsonStr);
            }
        }, firmwareVersionBean.getVersion(), firmwareVersionBean.getOtaType());
    }

    public CRPBleConnection connect(@NonNull MethodCall call, Context context) {
        String address = call.argument("address");
        bleDevice = CRPBleClient.create(context.getApplicationContext()).getBleDevice(address);
        if (bleDevice == null) {
            Log.d(TAG, "CRPBleDevice is null");
            return null;
        }
        CRPBleConnection connection = bleDevice.connect();
        if (connection == null) {
            Log.d(TAG, "Connection is null");
            return null;
        }

        for (BaseConnectionEventChannelHandler handler : connEveChlRegister.defaultConnHandlerList) {
            handler.setConnListener(connection, call);
        }
        return connection;
    }

    private void isConnected(MethodCall call, MethodChannel.Result result) {
        String address = call.argument("address");
        result.success(isConnected(address));
    }

    private boolean isConnected(String address) {
        CRPBleDevice mBleDevice = bleClient.getBleDevice(address);
        return mBleDevice != null && mBleDevice.isConnected();
    }

    private boolean checkBluetoothEnable() {
        if (!bleClient.isBluetoothEnable()) {
            Intent enableBtIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
            activity.startActivity(enableBtIntent);
        }

        return bleClient.isBluetoothEnable();
    }

    private void disconnect(MethodChannel.Result result) {
        if (bleDevice != null) {
            bleDevice.disconnect();
            result.success(true);
        }

        if (bleConnection != null) {
            bleConnection.close();
            bleConnection = null;
        }
    }

    private void closeGatt(MethodChannel.Result result) {
        if (bleConnection != null) {
            bleConnection.close();
            bleConnection = null;
            result.success(true);
        }
    }

    private void querySteps() {
        bleConnection.syncStep();
    }

    private void queryHistorySteps(MethodCall call) {
        Integer historyTimeType = call.argument("historyTimeType");
        historyTimeType = historyTimeType == null ?
                CRPPastTimeType.YESTERDAY_STEPS : historyTimeType;
        bleConnection.syncPastStep(historyTimeType.byteValue());
    }

    private void queryStepsDetail(MethodCall call) {
        Integer stepsDetailDateType = call.argument("stepsDetailDateType");
        stepsDetailDateType = stepsDetailDateType == null ?
                CRPStepsCategoryDateType.TODAY_STEPS_CATEGORY : stepsDetailDateType;
        bleConnection.queryStepsCategory(stepsDetailDateType.byteValue());
    }

    private void querySleep() {
        bleConnection.syncSleep();
    }

    private void queryRemSleep() {
        bleConnection.syncRemSleep();
    }

    private void queryHistorySleep(MethodCall call) {
        Integer historyTimeType = call.argument("historyTimeType");
        historyTimeType = historyTimeType == null ?
                CRPPastTimeType.YESTERDAY_SLEEP : historyTimeType;
        bleConnection.syncPastSleep(historyTimeType.byteValue());
    }

    private void startOTA(MethodCall call) {
        String jsonStr = call.argument("otaBean");
        OTABean otaBean = GsonUtils.get().fromJson(jsonStr,
                OTABean.class);
        OTAController otaController = new OTAController(otaBean.getType(), connEveChlRegister, call, bleConnection);
        otaController.startOTA();
    }

    private void abortOTA(MethodCall call) {
        Integer type = call.argument("type");
        type = type == null ? OTAController.START_DEFAULT_OTA : type;
        OTAController otaController = new OTAController(type, connEveChlRegister, call, bleConnection);
        otaController.abortOTA();
    }

    private void queryDeviceOtaStatus(MethodChannel.Result result) {
        bleConnection.queryDeviceDfuStatus(result::success);
    }

    private void queryHsOtaAddress(MethodChannel.Result result) {
        bleConnection.queryHsDfuAddress(address -> {
            Log.d(TAG, "onAddress: "+ address);
            result.success(address);
        });
    }

    private void enableHsOta() {
        bleConnection.enableHsDfu();
    }

    private void queryOtaType(MethodChannel.Result result) {
        bleConnection.queryDfuType(result::success);
    }

    private void queryUnitSystem(MethodChannel.Result result) {
        bleConnection.queryMetricSystem(result::success);
    }

    private void sendUnitSystem(MethodCall call) {
        Integer metricSystemType = call.argument("metricSystemType");
        metricSystemType = metricSystemType == null ?
                CRPMetricSystemType.METRIC_SYSTEM : metricSystemType;

        bleConnection.sendMetricSystem(metricSystemType.byteValue());
    }

    private void sendQuickView(MethodCall call) {
        Boolean quickViewState = call.argument("quickViewState");
        quickViewState = quickViewState != null && quickViewState;

        bleConnection.sendQuickView(quickViewState);
    }

    private void queryQuickView(MethodChannel.Result result) {
        bleConnection.queryQuickView(result::success);
    }

    private void sendQuickViewTime(MethodCall call) {
        String jsonStr = call.argument("periodTimeInfo");
        Log.d(TAG, jsonStr);
        CRPPeriodTimeInfo periodTimeInfo = GsonUtils.get().fromJson(jsonStr,
                CRPPeriodTimeInfo.class);

        bleConnection.sendQuickViewTime(periodTimeInfo);
    }

    private void queryQuickViewTime(MethodChannel.Result result) {
        bleConnection.queryQuickViewTime((int periodTimeType, CRPPeriodTimeInfo periodTimeInfo) -> {
            PeriodTimeBean quickViewTime = new PeriodTimeBean(periodTimeType,periodTimeInfo);
            String jsonStr = GsonUtils.get().toJson(quickViewTime, PeriodTimeBean.class);
            Log.i(TAG, "queryQuickViewTime: " + jsonStr );
            result.success(jsonStr);
        });
    }

    private void sendGoalSteps(MethodCall call) {
        Integer goalSteps = call.argument("goalSteps");
        goalSteps = goalSteps == null ? 8000 : goalSteps;

        bleConnection.sendGoalSteps(goalSteps);
    }

    private void queryGoalSteps(MethodChannel.Result result) {
        bleConnection.queryGoalStep(result::success);
    }

    private void sendDisplayWatchFace(MethodCall call) {
        Integer watchFaceType = call.argument("watchFaceType");
        watchFaceType = watchFaceType == null ? CRPWatchFaceType.FIRST_WATCH_FACE : watchFaceType;

        bleConnection.sendDisplayWatchFace(watchFaceType.byteValue());
    }

    private void queryDisplayWatchFace(MethodChannel.Result result) {
        bleConnection.queryDisplayWatchFace(result::success);
    }

    private void queryWatchFaceLayout(MethodChannel.Result result) {
        bleConnection.queryWatchFaceLayout(crpWatchFaceLayoutInfo -> {
            String jsonStr = GsonUtils.get().toJson(crpWatchFaceLayoutInfo, CRPWatchFaceLayoutInfo.class);
            result.success(jsonStr);
        });
    }

    private void sendWatchFaceLayout(MethodCall call) {
        String jsonStr = call.argument("watchFaceLayoutInfo");
        Log.d(TAG, jsonStr);
        CRPWatchFaceLayoutInfo watchFaceLayoutInfo = GsonUtils.get().fromJson(jsonStr, CRPWatchFaceLayoutInfo.class);

        bleConnection.sendWatchFaceLayout(watchFaceLayoutInfo);
    }

    private void sendWatchFaceBackground(MethodCall call) {
        BaseConnectionEventChannelHandler lazyEveChlHandler
                = connEveChlRegister.getLazyEveChlHandler(EVENT_FILE_TRANS);
        if (lazyEveChlHandler != null) {
            lazyEveChlHandler.setConnListener(bleConnection, call);
        }
    }

    private void querySupportWatchFace(MethodChannel.Result result) {
        bleConnection.querySupportWatchFace(supportWatchFaceInfo -> {
            String jsonStr = GsonUtils.get().toJson(supportWatchFaceInfo, CRPSupportWatchFaceInfo.class);
            result.success(jsonStr);
        });
    }

    private void queryWatchFaceStore(MethodCall call, MethodChannel.Result result) {
        String jsonStr = call.argument("watchFaceStoreBean");
        Log.d(TAG, jsonStr);
        WatchFaceStoreBean watchFaceLayoutInfo = GsonUtils.get().fromJson(jsonStr, WatchFaceStoreBean.class);
        Log.d(TAG,""+watchFaceLayoutInfo.getWatchFaceSupportList());
        Log.d(TAG,""+watchFaceLayoutInfo.getFirmwareVersion());

        bleConnection.queryWatchFaceStore(watchFaceLayoutInfo.getWatchFaceSupportList(), watchFaceLayoutInfo.getFirmwareVersion(),
                watchFaceLayoutInfo.getPageCount(), watchFaceLayoutInfo.getPageIndex(), new CRPDeviceWatchFaceStoreCallback() {
                    @Override
                    public void onWatchFaceStoreChange(CRPWatchFaceInfo watchFaceInfo) {
                        WatchFaceStoreListBean watchFaceStore = new WatchFaceStoreListBean(watchFaceInfo,"noError");
                        String jsonStr = GsonUtils.get().toJson(watchFaceStore, WatchFaceStoreListBean.class);
                        result.success(jsonStr);
                    }

                    @Override
                    public void onError(String error) {
                        CRPWatchFaceInfo watchFaceInfo = new CRPWatchFaceInfo(-1,-1,-1,new LinkedList<>());
                        WatchFaceStoreListBean watchFaceStore = new WatchFaceStoreListBean(watchFaceInfo,error);
                        String jsonStr = GsonUtils.get().toJson(watchFaceStore, WatchFaceStoreListBean.class);
                        result.success(jsonStr);
                    }
                });
    }

    private void queryWatchFaceOfID(MethodCall call, MethodChannel.Result result) {
        Integer id = call.argument("id");
        id = id == null ? -1: id;
        bleConnection.queryWatchFaceOfID(id, new CRPDeviceWatchFaceCallback() {
            @Override
            public void onWatchFaceChange(CRPWatchFaceInfo.WatchFaceBean watchFace) {
                int result_OK = 0;
                WatchFaceIDBean watchFaceID = new WatchFaceIDBean(result_OK);
                watchFaceID.setWatchFace(watchFace);
                watchFaceID.setError("");
                String jsonStr = GsonUtils.get().toJson(watchFaceID, WatchFaceIDBean.class);
                Log.i(TAG, "onWatchFaceChange: " + jsonStr);
                result.success(jsonStr);
            }

            @Override
            public void onError(String error) {
                int errorCode = 101;
                WatchFaceIDBean watchFaceID = new WatchFaceIDBean(errorCode);
                watchFaceID.setError(error);
                watchFaceID.setWatchFace(new CRPWatchFaceInfo.WatchFaceBean(-1,null,null));
                String jsonStr = GsonUtils.get().toJson(watchFaceID, WatchFaceIDBean.class);
                Log.i(TAG, "onError: " + jsonStr);
                result.success(jsonStr);
            }
        });
    }

    private void sendWatchFace(MethodCall call) {
        BaseConnectionEventChannelHandler lazyEveChlHandler
                = connEveChlRegister.getLazyEveChlHandler(EVENT_WF_FILE_TRANS);
        if (lazyEveChlHandler != null) {
            lazyEveChlHandler.setConnListener(bleConnection, call);
        }
    }

    private void sendAlarm(MethodCall call) {
        String jsonStr = call.argument("alarmClockInfo");
        Log.d(TAG, jsonStr);
        CRPAlarmClockInfo alarmClockInfo = GsonUtils.get().fromJson(jsonStr, CRPAlarmClockInfo.class);

        bleConnection.sendAlarmClock(alarmClockInfo);
    }

    private void queryAllAlarm(MethodChannel.Result result) {
        bleConnection.queryAllAlarmClock(list -> {
            List<String> jsonStr = new ArrayList<>();
            for (int i = 0; i < list.size(); i++) {
                jsonStr.add(GsonUtils.get().toJson(list.get(i), CRPAlarmClockInfo.class));
            }
            result.success(jsonStr.toString());
        });
    }

    private void queryLastDynamicRate(MethodCall call) {
        String type = call.argument("type");
        type = type == null ? CRPHistoryDynamicRateType.FIRST_HEART_RATE.toString() : type;

        bleConnection.queryLastDynamicRate(CRPHistoryDynamicRateType.valueOf(type));
    }

    private void enableTimingMeasureHeartRate(MethodCall call) {
        Integer interval = call.argument("interval");
        interval = interval == null ? 5 : interval;
        Log.d(TAG, ""+interval);
        bleConnection.enableTimingMeasureHeartRate(interval);
    }

    private void disableTimingMeasureHeartRate() {
        bleConnection.disableTimingMeasureHeartRate();
    }

    private void queryTimingMeasureHeartRate(MethodChannel.Result result) {
        bleConnection.queryTimingMeasureHeartRate(result::success);
    }

    private void queryTodayHeartRate(MethodCall call) {
        Integer heartRateType = call.argument("heartRateType");
        heartRateType = heartRateType == null ? CRPHeartRateType.ALL_DAY_HEART_RATE : heartRateType;
        Log.d(TAG, heartRateType+"");
        bleConnection.queryTodayHeartRate(heartRateType);
    }

    private void queryPastHeartRate() {
        bleConnection.queryPastHeartRate();
    }

    private void queryTrainingHeartRate() {
        bleConnection.queryMovementHeartRate();
    }

    private void startMeasureOnceHeartRate() {
        bleConnection.startMeasureOnceHeartRate();
    }

    private void stopMeasureOnceHeartRate() {
        bleConnection.stopMeasureOnceHeartRate();
    }

    private void queryHistoryHeartRate() {
        bleConnection.queryHistoryHeartRate();
    }

    //血压
    private void startMeasureBloodPressure() {
        bleConnection.startMeasureBloodPressure();
    }

    private void stopMeasureBloodPressure() {
        bleConnection.stopMeasureBloodPressure();
    }

    private void enableContinueBloodPressure() {
        bleConnection.enableContinueBloodPressure();
    }

    private void disableContinueBloodPressure() {
        bleConnection.disableContinueBloodPressure();
    }

    private void queryContinueBloodPressureState() {
        bleConnection.queryContinueBloodPressureState();
    }

    private void queryLast24HourBloodPressure() {
        bleConnection.queryLast24HourBloodPressure();
    }

    private void queryHistoryBloodPressure() {
        bleConnection.queryHistoryBloodPressure();
    }

    private void startMeasureBloodOxygen() {
        bleConnection.startMeasureBloodOxygen();
    }

    private void stopMeasureBloodOxygen() {
        bleConnection.stopMeasureBloodOxygen();
    }

    private void enableTimingMeasureBloodOxygen(MethodCall call) {
        Integer interval = call.argument("interval");
        interval = interval == null ? 5 : interval;
        Log.d(TAG, ""+interval);
        bleConnection.enableTimingMeasureBloodOxygen(interval);
    }

    private void disableTimingMeasureBloodOxygen() {
        bleConnection.disableTimingMeasureBloodOxygen();
    }

    private void queryTimingBloodOxygenMeasureState() {
        bleConnection.queryTimingBloodOxygenMeasureState();
    }

    private void queryTimingBloodOxygen(MethodCall call) {
        String bloodOxygenTimeType = call.argument("bloodOxygenTimeType");
        bloodOxygenTimeType = bloodOxygenTimeType == null ? CRPBloodOxygenTimeType.TODAY.toString() : bloodOxygenTimeType;
        Log.d(TAG,bloodOxygenTimeType);
        bleConnection.queryTimingBloodOxygen(CRPBloodOxygenTimeType.valueOf(bloodOxygenTimeType));
    }

    private void enableContinueBloodOxygen() {
        bleConnection.enableContinueBloodOxygen();
    }

    private void disableContinueBloodOxygen() {
        bleConnection.disableContinueBloodOxygen();
    }

    private void queryContinueBloodOxygenState() {
        bleConnection.queryContinueBloodOxygenState();
    }


    private void queryLast24HourBloodOxygen() {
        bleConnection.queryLast24HourBloodOxygen();
    }

    private void queryHistoryBloodOxygen() {
        bleConnection.queryHistoryBloodOxygen();
    }

    private void enterCameraView() {
        bleConnection.enterCameraView();
    }

    private void exitCameraView() {
        bleConnection.exitCameraView();
    }

    private void readDeviceRssi() {
        bleConnection.readDeviceRssi();
    }

    private void setECGChangeListener(MethodCall call) {
        BaseConnectionEventChannelHandler lazyEveChlHandler
                = connEveChlRegister.getLazyEveChlHandler(EVENT_ECG);
        if (lazyEveChlHandler != null) {
            lazyEveChlHandler.setConnListener(bleConnection, call);
        }
    }

    private void startECGMeasure() {
        bleConnection.startECGMeasure();
    }

    private void stopECGMeasure() {
        bleConnection.stopECGMeasure();
    }

    private void isNewECGMeasurementVersion(MethodChannel.Result result) {
        boolean newMeasurementVersion = bleConnection.isNewECGMeasurementVersion();

        Log.d(TAG, "isNewECGMeasurementVersion: " + newMeasurementVersion);
        result.success(newMeasurementVersion);

    }

    private void queryLastMeasureECGData() {
        bleConnection.queryLastMeasureECGData();
    }

    private void sendECGHeartRate(MethodCall call) {
        Integer heartRate = call.argument("heartRate");
        heartRate = heartRate == null ? 0 : heartRate;
        bleConnection.sendECGHeartRate(heartRate);
    }

    private void sendDeviceLanguage(MethodCall call) {
        Integer language = call.argument("language");
        language = language == null ? CRPDeviceLanguageType.LANGUAGE_CHINESE : language;
        // Simplified Chinese is set to the Chinese version, non-simplified Chinese is set to the international version
        byte version = CRPDeviceVersionType.INTERNATIONAL_EDITION;
        if (language == CRPDeviceLanguageType.LANGUAGE_CHINESE) {
            version = CRPDeviceVersionType.CHINESE_EDITION;
        }
        Log.d(TAG,""+version);
        Log.d(TAG, ""+language.byteValue());
        bleConnection.sendDeviceVersion(version);
        bleConnection.sendDeviceLanguage(language.byteValue());
    }

    private void queryDeviceLanguage(MethodChannel.Result result) {
        bleConnection.queryDeviceLanguage((int type, int[] languageType) -> {

            DeviceLanguageBean deviceLanguage = new DeviceLanguageBean(type,languageType);
            String jsonStr = GsonUtils.get().toJson(deviceLanguage, DeviceLanguageBean.class);
            result.success(jsonStr);
        });
    }

    private void sendMessage(MethodCall call) {
        String jsonStr = call.argument("messageInfo");
        Log.d(TAG, jsonStr);
        CRPMessageInfo messageInfo = GsonUtils.get().fromJson(jsonStr, CRPMessageInfo.class);
        bleConnection.sendMessage(messageInfo);
    }

    private void endCall() {
        bleConnection.sendCall0ffHook();
    }

    private void sendSedentaryReminder(MethodCall call) {
        Boolean sedentaryReminder = call.argument("sedentaryReminder");
        sedentaryReminder =   sedentaryReminder != null && sedentaryReminder;
        Log.d(TAG,""+sedentaryReminder);
        bleConnection.sendSedentaryReminder(sedentaryReminder);
    }

    private void querySedentaryReminder(MethodChannel.Result result) {
        bleConnection.querySedentaryReminder(result::success);
    }

    private void sendSedentaryReminderPeriod(MethodCall call) {
        String jsonStr = call.argument("sedentaryReminderPeriodInfo");
        Log.d(TAG, jsonStr);
        CRPSedentaryReminderPeriodInfo sedentaryReminderPeriodInfo =
                GsonUtils.get().fromJson(jsonStr, CRPSedentaryReminderPeriodInfo.class);
        bleConnection.sendSedentaryReminderPeriod(sedentaryReminderPeriodInfo);
    }

    private void querySedentaryReminderPeriod(MethodChannel.Result result) {
        bleConnection.querySedentaryReminderPeriod(sedentaryReminderPeriodInfo -> {
            String jsonStr = GsonUtils.get().toJson(sedentaryReminderPeriodInfo, CRPSedentaryReminderPeriodInfo.class);
            result.success(jsonStr);
        });
    }

    private void findDevice() {
        bleConnection.findDevice();
    }

    private void shutDown() {
        bleConnection.shutDown();
    }

    private void sendDoNotDisturbTime(MethodCall call) {
        String jsonStr = call.argument("periodTimeInfo");
        Log.d(TAG, jsonStr);
        CRPPeriodTimeInfo periodTimeInfo =
                GsonUtils.get().fromJson(jsonStr, CRPPeriodTimeInfo.class);
        bleConnection.sendDoNotDistrubTime(periodTimeInfo);
    }

    private void queryDoNotDisturbTime(MethodChannel.Result result){
        bleConnection.queryDoNotDistrubTime((periodTimeType, periodTimeInfo) -> {
            PeriodTimeBean doNotDisturbTime = new PeriodTimeBean(periodTimeType,periodTimeInfo);
            String jsonStr = GsonUtils.get().toJson(doNotDisturbTime, PeriodTimeBean.class);
            Log.i(TAG, "onPeriodTime: " + jsonStr);
            result.success(jsonStr);
        });
    }

    private void sendBreathingLight(MethodCall call) {
        Boolean breathingLight = call.argument("breathingLight");
        breathingLight = breathingLight != null && breathingLight;
        bleConnection.sendBreathingLight(breathingLight);
    }

    private void queryBreathingLight(MethodChannel.Result result) {
        bleConnection.queryBreathingLight(result::success);
    }

}
